Exception: Tuber::NotConnected
- Inherits:
-
RuntimeError
- Object
- RuntimeError
- Tuber::NotConnected
- Defined in:
- lib/tuber/errors.rb
Overview
Raises when a beanstalkd instance is no longer accessible.
Raises when a beanstalkd instance is no longer accessible.